from xml.etree.ElementTree import ElementTree
from optparse import OptionParser
import StringIO
import string

def retrieveAttrsStr(attrDict):
  attrs = "".join([' %s="%s"' % \
          (key, value) for key, value in attrDict.items()])
  return attrs

def shouldModifyHtml(tree):
  geneTag = tree.find("body/table/tr/td/a")
  if geneTag.text.isdigit():
    toBeRemoved = True
  elif geneTag.text.endswith("at")==False:
    toBeRemoved = True
  else:
    toBeRemoved = False
  return toBeRemoved

def html2Txt(htmlFilename, txtOutFilename):
  htmlTempContent = file(htmlFilename, 'r').readlines()
  htmlContent = makeWellForm(htmlTempContent)
  txtOutFile = file(txtOutFilename, 'w')
 
  outTxt = ""
  headerList = []
  tree = ElementTree()
  tree.parse(StringIO.StringIO(htmlContent))
  for row in tree.find("body/table").getchildren():
    for header in row.getchildren():
      if header.text == None:
	if len(header.getchildren()) != 0:
          outTxt += header.getchildren()[0].text + "\t"
        else:
          outTxt += "\t"
      elif header.tag == "th":
        headerList.append(header.text)
      else:
        outTxt += header.text + "\t"
    outTxt += "\n"

  outTxt = "\t".join(headerList) + outTxt
  txtOutFile.write(outTxt)
  txtOutFile.close()


def constructTable(tree):
  outHtml = ["<table border=\"2\">\n<tr>\n"]
  for row in tree.find("body/table").getchildren():
    if row.getchildren()[0].text == "Probe":
      outHtml.append("<th>Gene ID</th>\n")
    else: #<td> tag here
      tdAddr = retrieveAttrsStr(row.getchildren()[0].attrib)
      children  = row.getchildren()[0].getchildren()
      childAttr= retrieveAttrsStr(children[0].attrib)
      tag = row.getchildren()[0].tag
      outHtml.append("<tr>\n<%s><%s>%s</%s></%s>\n" % \
                (tag + tdAddr, children[0].tag + childAttr, \
                children[0].text, children[0].tag, tag))
    outHtml.append(string.join(map(lambda x: "<%s>%s</%s>\n" % \
            (x.tag + retrieveAttrsStr(x.attrib), x.text, x.tag), row.getchildren()[-2:]), ""))
    outHtml.append("</tr>\n")
  outHtml.append("</table>\n")
  return outHtml

def constructGeneHtml(tree, outfilename):
  tableHtml = constructTable(tree)

  outfile = open(outfilename, 'w')
  outfile.write("<html>\n<head>\n  <title>Target Genes</title>\n" \
	      + "  <meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n" \
	      + "  <style type=\"text/css\"></style>\n" \
	      + "  <script language=\"JavaScript\"></script>\n" \
	      + "</head>\n" \
	      + "<body bgcolor=\"#FFFFFF\">\n" \
	      + "<h1 align=\"center\">Target Genes</h1>\n") 
  outfile.write("".join(tableHtml))
  
  pElement = tree.find("body/p")
  pTagHtml = "<%s>%s</%s>\n" % (pElement.tag, pElement.text, pElement.tag)
  print "".join(tableHtml)
  print pTagHtml
  outfile.write(pTagHtml)
  outfile.write("</body>\n</html>")
  outfile.close()

def makeWellForm(htmlFile):
  html = ""
  for line in htmlFile:
    if line.find("<meta")== -1:
      html += line.replace("&nbsp;", "").replace("&", "")
  return html




def main():
  parser = OptionParser()
  parser.add_option("-f", "--file", dest="filename", \
		    help="Modify the content of the Affymetrix Analysis HTML", \
		    metavar="FILE")
  (options, args) = parser.parse_args()
    
  infile = open(options.filename)
  html = makeWellForm(infile)
  infile.close()

  tree = ElementTree()
  tree.parse(StringIO.StringIO(html))
  toModifyHtml = shouldModifyHtml(tree)

  if toModifyHtml == True:
    constructGeneHtml(tree, "temp.html")
    print "toModifyHtml: True"

  print "b4 html2Txt"
  html2Txt("temp.html", "temp.txt")

if __name__ == "__main__":
    main()

