/* NOTE: MODIFIED FROM ORIGINAL! */

(function($) {
    function fixTitle($ele) {
        if ($ele.attr('title') || typeof($ele.attr('original-title')) != 'string') {
            $ele.attr('original-title', $ele.attr('title') || '').removeAttr('title');
        }
    }
    
    $.fn.tipsy = function(options) {

        options = $.extend({}, $.fn.tipsy.defaults, options);
        
        return this.each(function() {
            
            fixTitle($(this));
            var opts = $.fn.tipsy.elementOptions(this, options);
            var timeout = null;
            
            $(this).hover(function() {
                var self = this;
                timeout = setTimeout(function() {
                    $.data(self, 'cancel.tipsy', true);

                    var tip = $.data(self, 'active.tipsy');
                    if (!tip) {
                        tip = $('<div class="tipsy"><div class="tipsy-inner"/></div>');
                        tip.css({position: 'absolute', zIndex: 100000});
                        $.data(self, 'active.tipsy', tip);
                    }

                    fixTitle($(self));

                    var title;
                    if (typeof opts.title == 'string') {
                        title = $(self).attr(opts.title == 'title' ? 'original-title' : opts.title);
                    } else if (typeof opts.title == 'function') {
                        title = opts.title.call(self);
                    }

                    tip.find('.tipsy-inner')[opts.html ? 'html' : 'text'](title || opts.fallback);


                    var pos = $.extend({}, $(self).offset(), {width: self.offsetWidth, height: self.offsetHeight});
                    tip.get(0).className = 'tipsy'; // reset classname in case of dynamic gravity
                    tip.remove().css({top: 0, left: 0, visibility: 'hidden', display: 'block'}).appendTo(document.body);

                    tip.css( { width: tip.width() + 1, height: tip.height() } );
                                        
                    var actualWidth = tip[0].offsetWidth, actualHeight = tip[0].offsetHeight;
                    var gravity = (typeof opts.gravity == 'function') ? opts.gravity.call(self) : opts.gravity;

                    var top, left;
                    switch (gravity.charAt(0)) {                      
                        case 'n':
                            top = pos.top + pos.height;
                            left = pos.left + pos.width / 2 - actualWidth / 2;
                            tip.addClass('tipsy-north');
                            break;
                        case 's':
                            top = pos.top - actualHeight;
                            left = pos.left + pos.width / 2 - actualWidth / 2;
                            tip.addClass('tipsy-south');
                            break;
                        case 'e':
                            top = pos.top + pos.height / 2 - actualHeight / 2;
                            left = pos.left - actualWidth;
                            tip.addClass('tipsy-east');
                            break;
                        case 'w':
                            top = pos.top + pos.height / 2 - actualHeight / 2;
                            left = pos.left + pos.width;
                            tip.addClass('tipsy-west');
                            break;
                    }
                    // Shift if off screen
                    var w = $(window);
                    
                    // If off the top of the screen, flip
                    if ( top < w.scrollTop() && gravity.charAt( 0 ) == 's' ) {
                        top = pos.top + pos.height;
                        gravity = 'north';
                        tip.removeClass('tipsy-south').addClass('tipsy-north');
                    }
                    
                    // If off bottom, just shift for now
                    top = Math.min( top, w.scrollTop() + w.height() - tip.outerHeight() );
                    
                    
                    // Shift left or right
                    var left_shift = 0;
                    if ( left < w.scrollLeft() ) {
                        left_shift = left - w.scrollLeft();
                    }
                    var t = w.scrollLeft() + w.width() - tip.outerWidth();
                    if ( left > t ) {
                        left_shift = left - t;
                    }

                    left -= left_shift;
                    
                    tip.css( { left: left, top: top } );
                    
                    // Shift background to center over element (not implemented for east/west)
                    switch (gravity.charAt(0)) {                      
                        case 'n':
                            tip.css( 'background-position', - ( 250 - tip.outerWidth() / 2 ) + left_shift + "px top" );
                            break;
                        case 's':
                            tip.css( 'background-position', - ( 250 - tip.outerWidth() / 2 ) + left_shift + "px bottom" );
                            break;
                        case 'e':
                            break;
                        case 'w':
                            break;
                    }
                    
                    if (opts.fade) {
                        tip.stop().css({opacity: 0, display: 'block', visibility: 'visible'}).animate({opacity: opts.opacity});
                    } else {
                        tip.css({visibility: 'visible', opacity: opts.opacity});
                    }
                }, opts.delayIn);

            }, function() {
                $.data(this, 'cancel.tipsy', false);
                var self = this;
                clearTimeout(timeout);
                setTimeout(function() {
                    if ($.data(this, 'cancel.tipsy')) { return; }
                    var tip = $.data(self, 'active.tipsy');
                    if (opts.fade) {
                        tip.stop().fadeOut(function() { $(this).remove(); });
                    } else if (tip) {
                        tip.remove();
                    }
                }, opts.delayOut);

            });
            
        });
        
    };
    
    // Overwrite this method to provide options on a per-element basis.
    // For example, you could store the gravity in a 'tipsy-gravity' attribute:
    // return $.extend({}, options, {gravity: $(ele).attr('tipsy-gravity') || 'n' });
    // (remember - do not modify 'options' in place!)
    $.fn.tipsy.elementOptions = function(ele, options) {
        return $.metadata ? $.extend({}, options, $(ele).metadata()) : options;
    };
    
    $.fn.tipsy.defaults = {
        delayIn: 0,
        delayOut: 100,
        fade: false,
        fallback: '',
        gravity: 'n',
        html: false,
        opacity: 0.8,
        title: 'title'
    };
    
    $.fn.tipsy.autoNS = function() {
        return $(this).offset().top > ($(document).scrollTop() + $(window).height() / 2) ? 's' : 'n';
    };
    
    $.fn.tipsy.autoWE = function() {
        return $(this).offset().left > ($(document).scrollLeft() + $(window).width() / 2) ? 'e' : 'w';
    };
    
})(jQuery);
