"""
corrtf.py

Computes correlation between genes using their expression values.

"""
import sys, string, os, tempfile, zipfile, subprocess

rprog="""corr=function(infile,outfile,geneName,cutoff)
#  rcall = "%s('%s','%s','%s','%s')" % (corr,infile,outfile,geneName,cutoff)
{
    sink(file=file("/dev/null", "w"), type="message")
    sink(file=file("/dev/null", "w"), type="output")
    library(affy)
    library(annaffy)
    x <- load(infile)
    eset <- get(x)
    db <- annotation(eset)
    db <- paste(db, "db", sep=".")
    if (db  == "fly.db0.db"){db = "fly.db0"}
    probes <- featureNames(eset)
    symbols <- aafSymbol(probes, db)
    mappedProbes <- probes[which(symbols == geneName)]
    chosenProbe <- mappedProbes[1]

    m <- exprs(eset)
    m.t <- t(m)
    x <- tryCatch(m.t[,chosenProbe], error = function(chosenProbe) { "Bad" })
    if (class(x) == "character" && x == "Bad") {
        write.table(c(), file=outfile)
    } else {
        co <- cor(x, m.t)
        sorted <- sort(co[1,], decreasing=TRUE)
        sorted <- sorted[sorted > 0]
        sorted <- sorted[sorted > cutoff]
        write.table(sorted, file=outfile)
    }
}
"""

def main():
    """called as 
<command interpreter="python">
corrtf.py '$i' '$db' '$geneName' '$y' '$species' '$coA' '$coR' '$tf' '$tr' '$logmeta'
</command>
    """
    nparm = 10
    appname = sys.argv[0]
    if (len(sys.argv) < nparm):
        print '%s needs %d parameters - given %d as %s' % (appname,nparm,len(sys.argv),';'.join(sys.argv))
        print 'usage: %s eSet db geneName cutoff species coA coR tf tr outfile' % (appname)
        sys.exit(1)
        # short command line error
    i = sys.argv[1].strip()
    db = sys.argv[2].strip()
    geneName = sys.argv[3].strip()
    if (geneName == ''):
        raise Exception("Gene symbol cannot be empty")
    cutoff = float(sys.argv[4].strip())
    species = sys.argv[5].strip()
    coA = sys.argv[6].strip()
    coR = sys.argv[7].strip()
    tf= sys.argv[8].strip()
    tr = sys.argv[9].strip()
    outfile = sys.argv[10].strip()

    rprogname = 'corr'
    rcall = "%s('%s','%s','%s',%f)" % (rprogname,i,outfile,geneName,cutoff)
    print '##rcall=',rcall
    p = subprocess.Popen("R --vanilla", shell=True, executable="/bin/bash", stdin=subprocess.PIPE)
    p.communicate(rprog + "\n" + rcall)

    print "### cwd\t%s" % (appname.rsplit('/', 1)[0])

if __name__ == "__main__":
    main()

