"""
exptf.py
A galaxy tool to find the top-n highest expressed TFs
"""
import sys, string, os, tempfile, zipfile

def findHighestExpressedTFs(cwd, infpath, i, cutoff, filterSpecies, coA, coR, tf, tr, logfname):
    #read infpath
    #drop header
    #genes = extract 1st column (genes) in vector
    #exp_val = extract n-th column in vector
    #map(exp_val, gene)
    #sort exp_val
    #loop sorted_exp_val and print gene until sorted_exp_val < cutoff
    #filter by coA, coR, tf, tr

    f = file(infpath, 'r')
    lines = f.readlines()
    f.close()

    header = lines[0]
    columns = header.split()
    column = columns[i - 1]
    firstLine = column + '\texp_value'
    lines = lines[1:]

    genes = []
    exp_val = []
    nameMappings = {}
    valueMappings = {}

    for (line) in lines:
        columns = line.strip().split()
        geneName = columns[0].strip('"')
        value = float(columns[i])
        genes.append(geneName)
        exp_val.append(value)
        nameMappings[value] = geneName
        valueMappings[value] = columns[i]

    exp_val.sort()
    exp_val.reverse()

    header = None
    hasFilter = False
    if (filterSpecies <> 'Nil'):
        if (coA == 'T'):
            (header,coAMap) = getMap(os.path.join(cwd, 'coA' + filterSpecies))
            hasFilter = True
        if (coR == 'T'):
            (header,coRMap) = getMap(os.path.join(cwd, 'coR' + filterSpecies))
            hasFilter = True
        if (tf == 'T'):
            (header,tfMap) = getMap(os.path.join(cwd, 'tf' + filterSpecies))
            hasFilter = True
        if (tr == 'T'):
            (header,trMap) = getMap(os.path.join(cwd, 'tr' + filterSpecies))
            hasFilter = True

    f = file(logfname, 'w')
    if (header <> None):
        firstLine += '\t' + header
    else:
        firstLine += '\n'
    f.write(firstLine)
    for (v) in exp_val:
        if (v > cutoff):
            geneSymbol = nameMappings[v]
            s = None
            if (hasFilter):
                if (coA == 'T' and geneSymbol in coAMap):
                    s = "%s\t%s\t%s" % (geneSymbol, valueMappings[v], coAMap[geneSymbol])
                elif (coR == 'T' and geneSymbol in coRMap):
                    s = "%s\t%s\t%s" % (geneSymbol, valueMappings[v], coRMap[geneSymbol])
                elif (tf == 'T' and geneSymbol in tfMap):
                    s = "%s\t%s\t%s" % (geneSymbol, valueMappings[v], tfMap[geneSymbol])
                elif (tr == 'T' and geneSymbol in trMap):
                    s = "%s\t%s\t%s" % (geneSymbol, valueMappings[v], trMap[geneSymbol])

                if (s <> None):
                    f.write(s)
            else:
                f.write("%s\t%s\n" % (geneSymbol, valueMappings[v]))

    f.close()

def getMap(which):
    if (not os.path.isfile(which)):
        return (which, {})

    f = file(which, 'r')
    try:
        lines = f.readlines()
    finally:
        f.close()

    header = lines[0]
    headerTokens = header.split('\t')
    header = "\t".join(headerTokens[1:])
    lines = lines[1:]
    map = {}
    for (line) in lines:
        tokens = line.split('\t')
        map[tokens[0]] = '\t'.join(tokens[1:])

    return (header,map)

def main():
    """called as 
<command interpreter="python">
exptf.py '$expVal' '$i' '$cutoff' '$filterSpecies' '$coactivator' '$corepressor' '$tf' '$tr' '$logmeta'
</command>
    """
    nparm = 9
    appname = sys.argv[0]
    if (len(sys.argv) < nparm):
        print '%s needs %d parameters - given %d as %s' % (appname,nparm,len(sys.argv),';'.join(sys.argv))
        print 'usage: %s expression_values n cut_off filterSpecies coA coR tf tr out_file' % (appname)
        sys.exit(1)
        # short command line error
    infpath = sys.argv[1].strip()
    i = int(sys.argv[2].strip())
    cutoff = float(sys.argv[3].strip())
    filterSpecies = sys.argv[4].strip()
    coA = sys.argv[5].strip()
    coR = sys.argv[6].strip()
    tf = sys.argv[7].strip()
    tr = sys.argv[8].strip()
    logfname = sys.argv[9].strip()

    findHighestExpressedTFs(appname.rsplit('/', 1)[0], infpath, i, cutoff, filterSpecies, coA, coR, tf, tr, logfname)

if __name__ == "__main__":
    main()

